/*****************************************************************************/
/******** MMWS PROCEDURE FOR MULTIVALUED NOMINAL TREATMENT ********************/
/*****************************************************************************/

*** Sample: 3091 Language minority children
*** Outcome: t2learn
*** Binary treatment: dmellsrvc 
*** Multi-valued treatment: ellservc (with four categories: pullout, inclass, biingual, and noservice)
*** Pretreatment covariates for the exercise:
****** hmlang(with 3 category: missing, English, non-English)
****** race (with six categories: WHITE, BLACK, HISPANIC, ASIAN, NATIVE, OTHRACE)
****** cc1mscale, dmmsc1msor 
******  cwksesl, dmmsses  

u "C:\AERA 2013 MMWS workshop\AERA 2013 MMWS Data example\K_ELL_T2LEARN_exercise", clear
* Drop cases with no treatment information
drop if ellservc==-9
* Drop the variables that this program will generate
drop pr_* lgp_* support4 mmws

** Dummy out the analysis covariates;
gen enghmlang = hmlang==1
gen m_hmlang = hmlang==0

gen othrace = race==0
gen white = race==1
gen black = race==2
gen asian = race==4
gen native = race==5

****************************************************************
*** Step 1: Estimate Propensity Scores for ELLSERVC
**************************************************************** 

*** Multinomial logistic regression 
mlogit ellservc i.hmlang i.race cc1mscale dmmsc1msor cwksesl dmmsses 
predict pr_noservice, outcome(0)
predict pr_pullout, outcome(1)
predict pr_inclass, outcome(2)
predict pr_bilingual, outcome(3)

gen lgp_noservice = logit(pr_noservice)
gen lgp_pullout = logit(pr_pullout)
gen lgp_inclass = logit(pr_inclass)
gen lgp_bilingual = logit(pr_bilingual)

****************************************************************
*** Step 2: Identify common support
**************************************************************** 
global lgp "lgp_*"
foreach var of varlist $lgp {
**Find the standard deviation over the whole sample;
egen std`var'=sd(`var')
**Calculate the 20% swing;
gen sds`var'=0.2*std`var'
**Find the minimum values by treatment groups;
bysort ellservc: egen mins`var'=min(`var')
**Find the max of those minimum values;
egen min`var'=max(mins`var')
**Find the maximum values by treatment group;
bysort ellservc: egen maxs`var'=max(`var')
**Find the min of those maximum values;
egen max`var'=min(maxs`var')
**Require that a given score must above the minimum and below the maximum, with the 20% SD swing;
gen support`var'= (`var'>= (min`var'- sds`var') & `var'<= max`var'+ sds`var')
}

*Create a variable that indicates how many treatments in which the student falls into the common support
egen support = rowtotal(support*)
ta support

** Keep only the students who are in the common support for all four treatments
keep if support==4
** Drop extraneous variables
drop min* max* support* std* sds* 

*** 3019 students with valid treatment information
*** Among them, 160 students excluded from common support
*** In the analytic sample of 2859 students
****** 1186 had no service
****** 422 had pull-out
****** 408 had in-class
****** 843 had bilingual services 

****************************************************************
*** Step 3: Conduct Propensity Score Stratification
**************************************************************** 
****************************************************************
*** Step 4: Compute MMWS
**************************************************************** 

*** Stratify the analytic sample into six strata based on each logit score 
** Note that this distributes the observations slightly differently than in SPSS, which results in very slight differences in the mmws
xtile strata=lgp_noservice, n(6)
*** Find the proportion of obs. within a strata that actually receive the treatment
bysort strata: egen mn=mean(noservice)
*** Take the inverse of that proportion
gen iprws = 1/mn
*** Find the overall proportion of treatment within the whole sample
egen gprwws=mean(noservice) 
*** Calculate the weight for those observations that received the treatment
gen mmws = iprws*gprwws if noserv==1   
drop strata mn iprws gprwws

*** Pull-Out ESL
xtile strata=lgp_pullout, n(6)
*** Find the proportion of obs. within a strata that actually receive the treatment
bysort strata: egen mn=mean(pullout)
*** Take the inverse of that proportion
gen iprws = 1/mn
*** Find the overall proportion of treatment within the whole sample
egen gprwws=mean(pullout) 
*** Calculate the weight for those observations that received the treatment
replace mmws = iprws*gprwws if pullout==1   
drop strata mn iprws gprwws

*** In-Class ESL
xtile strata=lgp_inclass, n(6)
*** Find the proportion of obs. within a strata that actually receive the treatment
bysort strata: egen mn=mean(inclass)
*** Take the inverse of that proportion
gen iprws = 1/mn
*** Find the overall proportion of treatment within the whole sample
egen gprwws=mean(inclass) 
*** Calculate the weight for those observations that received the treatment
replace mmws = iprws*gprwws if inclass==1   
drop strata mn iprws gprwws


*** Bilingual
xtile strata=lgp_bilingual, n(6)
*** Find the proportion of obs. within a strata that actually receive the treatment
bysort strata: egen mn=mean(bilingual)
*** Take the inverse of that proportion
gen iprws = 1/mn
*** Find the overall proportion of treatment within the whole sample
egen gprwws=mean(bilingual) 
*** Calculate the weight for those observations that received the treatment
replace mmws = iprws*gprwws if bilingual==1   
drop strata mn iprws gprwws

sum mmws


****************************************************************
*** Step 5: Check Balance
****************************************************************

foreach var of varlist $lgp {
anova `var' pullout inclass bilingual  
anova `var' pullout inclass bilingual [aw=mmws] 
}

*** CHECK FOR BALANCE ON THE COVARIATES ***
global covars "enghmlang m_hmlang othrace white black asian native cc1mscale dmmsc1msor cwksesl dmmsses"
foreach var of varlist $covars {
anova `var' pullout inclass bilingual  
anova `var' pullout inclass bilingual [aw=mmws] 
}

****************************************************************
*** Step 6: Estimate Treatment Effects
****************************************************************

anova t2learn pullout inclass biling 
anova t2learn pullout inclass biling [aw=mmws]
