/*****************************************************************************/
/******************* MMW PROCEDURE FOR BINARY TREATMENT *****************/
/*****************************************************************************/

libname MW "C:\AERA 2013 MMWS workshop\AERA 2013 MMWS Data Example";


*** Sample: 3091 Language minority children;
*** Outcome: T2LEARN;
*** Binary treatment: DMELLSRVC;
*** Pretreatment covariates for the exercise:
    HMLANG (with 3 category: missing, English, non-English)
    RACE (with six categories: WHITE, BLACK, HISPANIC, ASIAN, NATIVE, OTHRACE)
    CC1MSCALE, DMMSC1MSOR
    CWKSESL, DMMSSES;


*** remove cases without valid treatment info;

data sampledata;
  set MW.K_ELL_T2LEARN_exercise;
  if DMELLSRVC=. then delete;
run;

*------------------------------------------------------------------------------;
* step 1 Estimate Propensity Score;
*------------------------------------------------------------------------------;

/* estimate propensity score */

proc logistic descending data=sampledata;
  class dmmsc1msor dmmsses HMLANG race/param=ref;
  model DMELLSRVC=cc1mscale cwksesl dmmsc1msor dmmsses HMLANG race;
  output out=pred predprobs=(individual);
run;

/* convert the propensity score onto a logit scale */

data propensity;
  set pred;
    lgp_z0=log(IP_0/(1-IP_0));
    lgp_z1=log(IP_1/(1-IP_1));
run;

*------------------------------------------------------------------------------;
* step 2 Identify Common Support;
*------------------------------------------------------------------------------;

proc sql;
    create table temp1 as
    select *, min(lgp_z1)as min1, max(lgp_z1)as max1
    from propensity
    group by DMELLSRVC;
quit;

proc sql;
    create table temp2 as
    select *, max(min1)as lbd, min(max1) as ubd, std(lgp_z1) as lgst
    from temp1;
quit;

data valid; /* define valid sample (use a caliper of 20% of one std) */
    set temp2;
    if lgp_z1<lbd-0.2*lgst or lgp_z1>ubd+0.2*lgst then delete;
run;


*** 3019 students with valid treatment information;
*** 6 students exclud from common support;
*** 3013 students in the analytic sample:
    1316 had no service
    1697 had ELL service;


*------------------------------------------------------------------------------;
* step 3 Conduct Propensity Score Stratification;
*------------------------------------------------------------------------------;

/*stratify the valid sample into 9 strata based on the logit score*/

proc univariate noprint data=valid;
   var lgp_z1;
   output out=pctz pctlpts=0 to 100 by 11.11 pctlpre=pct;
run;

proc print data=pctz;
run;

data valid2;
  set valid;
  zst=.;
  if lgp_z1 <= -1.30149 then zst=1;
  else if -1.30149 <lgp_z1<=-0.56852  then zst=2;
  else if -0.56852 <lgp_z1<=-0.072811 then zst=3;
  else if -0.072811<lgp_z1<= 0.32476  then zst=4;
  else if  0.32476 <lgp_z1<= 0.67603  then zst=5;
  else if  0.67603 <lgp_z1<= 0.94621  then zst=6;
  else if  0.94621 <lgp_z1<= 1.17702  then zst=7;
  else if  1.17702 <lgp_z1<= 1.45299  then zst=8;
  else zst=9;
run;

proc tabulate data=valid2;
  class DMELLSRVC zst;
  var lgp_z1;
  table zst all,lgp_z1*DMELLSRVC* (N Mean STDDEV) /RTS=10;
run;

*------------------------------------------------------------------------------;
* step 4 Compute MMW;
*------------------------------------------------------------------------------;

proc sql;
  create table temp1 as
  select *, count(*) as nz
  from valid2
  group by DMELLSRVC;
quit;

proc sql;
  create table temp2 as
  select *, count(*) as total, nz/count(*) as pz
  from temp1;
quit;

proc sql;
  create table temp3 as
  select *, count(*) as ns
  from temp2
  group by zst;
quit;

proc sql;
  create table temp4 as
  select *, count(*) as nsz
  from temp3
  group by zst, DMELLSRVC;
quit;

data MW.bmmw;
  set temp4;
  mmw=ns*pz/nsz;
run;

proc means data=MW.bmmw mean std n;
  class zst DMELLSRVC ;
  var mmw;
run;


*------------------------------------------------------------------------------;
* step 5 Check Balance;
*------------------------------------------------------------------------------;

/* check balance on logit score*/

proc reg data=MW.bmmw;
  model lgp_z1= DMELLSRVC;
  weight mmw;
run;


/* check balance on continuous pretreatment variables*/

proc reg data=MW.mmw;
  model cc1mscale=DMELLSRVC;
  weight mmw;
run;

proc reg data=MW.mmw;
  model cwksesl=DMELLSRVC;
  weight mmw;
run;


/* check balance on binary pretreatment variables*/

proc logistic data=MW.mmw;
  model dmmsc1msor (EVENT='1')=DMELLSRVC;
  weight mmw;
run;

proc logistic data=MW.mmw;
  model dmmsses (EVENT='1')=DMELLSRVC;
  weight mmw;
run;

/* check balance on multi-category pretreatment variables*/

proc freq data=MW.mmw;
  weight mmw;
  tables HMLANG*DMELLSRVC/chisq;
run;

proc freq data=MW.mmw;
  weight mmw;
  tables race*DMELLSRVC/chisq;
run;

*------------------------------------------------------------------------------;
* step 6 Estimate Treatment Effects;
*------------------------------------------------------------------------------;

/*naive analysis*/

proc glm data=MW.bmmw;
  class DMELLSRVC;
  model t2learn=DMELLSRVC;
run;

/*weighted analysis*/

proc glm data=MW.bmmw;
  class DMELLSRVC;
  model t2learn=DMELLSRVC;
  weight mmw;
run;

