/*****************************************************************************/
/******************* MMW PROCEDURE FOR MULTIVALUED TREATMENT *****************/
/*****************************************************************************/

libname MW "C:\AERA 2013 MMWS workshop\AERA 2013 MMWS Data Example";


*** Sample: 3091 Language minority children;
*** Outcome: T2LEARN;
*** Multi-valued treatment:
    ELLSERVC (with four categories: pullout[1], inclass[2], biingual[3], noservice[0]);
*** Pretreatment covariates for the exercise:
    HMLANG (with 3 category: missing, English, non-English)
    RACE (with six categories: WHITE, BLACK, HISPANIC, ASIAN, NATIVE, OTHRACE)
    CC1MSCALE, DMMSC1MSOR
    CWKSESL, DMMSSES;



*** remove cases without valid treatment info;

data sampledata;
  set MW.K_ELL_T2LEARN_exercise;
  if ELLSERVC=-9 then delete;
run;


*** identify the range of the treatment and code it into dummy indicators;

%let z=ELLSERVC; /*specify treatment variable*/
proc sql noprint;
 select distinct min(&z),max(&z)
 into :bcount,:ecount
 from sampledata;
quit;

%macro TR (indata,outdata);
data &outdata;
  set &indata;
  %do n=&bcount %to &ecount;
    if &z=&n then z&n=1;
        else z&n=0;
  %end;
run;
%mend TR;

%TR (indata=sampledata,      /* specify input dataset */
     outdata=tredata);       /* specify output dataset */


*------------------------------------------------------------------------------;
* step 1 Estimate Propensity Score;
*------------------------------------------------------------------------------;

%macro PS (indata, conv, biv, cav, outdata);
/* estimate propensity score */
proc logistic descending data=&indata;
  class &biv &cav/param=ref;
  model &z=&conv &biv &cav/link=glogit;
  output out=pred predprobs=(individual);
run;

/* convert the propensity score onto a logit scale */
data &outdata;
  set pred;
  %do n=&bcount %to &ecount;
    lgp_z&n=log(IP_&n/(1-IP_&n));
  %end;
run;
%mend PS;

%PS (indata=tredata,                               /* specify input dataset */
     conv=cc1mscale cwksesl,                       /* specify continuous covariates */
     biv=dmmsc1msor dmmsses,                       /* specify binary covariates including dummy missing indicators */
     cav=HMLANG race,                              /* specify categorical covariates */
     outdata=propensity);                          /* specify output dataset */


*------------------------------------------------------------------------------;
* step 2 Identify Common Support;
*------------------------------------------------------------------------------;

%macro CS (indata, outdata);
%let i=&bcount;
  %do %until (&i>&ecount);
    proc sql;
      create table temp&i as
      select *, min(lgp_z&i)as min&i, max(lgp_z&i)as max&i
      from &indata
      group by &z;
    quit;

    proc sql noprint;
      select max(min&i), min(max&i), std(lgp_z&i)
      into :lbound&i,:ubound&i,:std&i
      from temp&i;
    quit;

    data &indata; /* define valid sample (use a caliper of 20% of one std) */
      set &indata;
          if lgp_z&i<(&&lbound&i)-0.2*(&&std&i)or lgp_z&i >(&&ubound&i)+0.2*(&&std&i)then NV=1;
    run;
  %let i=%eval(&i+1);
%end;

data &outdata;
  set &indata;
  if NV=1 then delete;
run;
%mend CS;

%CS (indata=propensity, outdata=valid); /* specify input & output datasets */


*** 3019 students with valid treatment information;
*** 160 students exclud from common support;
*** 2859 students in the analytic sample:
    1186 had no service
    422 had pull-out
    408 had in-class
    843 had bilingual services;


*------------------------------------------------------------------------------;
* step 3 Conduct Propensity Score Stratification;
*------------------------------------------------------------------------------;

/*stratify the valid sample into six strata based on each logit score*/

%macro STA (mydata);
%let i=&bcount;
  %do %until (&i>&ecount) ;
    proc univariate noprint data=&mydata;
      var lgp_z&i;
      output out=pctz&i pctlpts=0 to 100 by 16.67 pctlpre=pct;
    run;

    proc sql noprint;
      select pct16_67, pct33_34, pct50_01, pct66_68, pct83_35
      into :s1&i, :s2&i, :s3&i, :s4&i, :s5&i
      from pctz&i;
    quit;

    data &mydata;
      set &mydata;
      if lgp_z&i <= &&s1&i then z&i.st=1;
        else if &&s1&i < lgp_z&i <= &&s2&i then z&i.st=2;
        else if &&s2&i < lgp_z&i <= &&s3&i then z&i.st=3;
        else if &&s3&i < lgp_z&i <= &&s4&i then z&i.st=4;
        else if &&s4&i < lgp_z&i <= &&s5&i then z&i.st=5;
        else z&i.st=6;
      %do m=1 %to 6;
        z&i.st&m=0;
        if z&i.st=&m then z&i.st&m=1;
          %end;
    run;

    proc tabulate data=&mydata;
        class z&i z&i.st;
        var lgp_z&i;
        table z&i.st all,lgp_z&i*z&i* (N Mean STDDEV) /RTS=10;
    run;

  %let i=%eval(&i+1);
%end;

%mend STA;

%STA (mydata=valid);   /* specify the dataset to use */


*------------------------------------------------------------------------------;
* step 4 Compute MMW;
*------------------------------------------------------------------------------;

%macro MMW (indata, outdata);
proc sql;
  create table temp1 as
  select *, count(*) as nz
  from &indata
  group by &z;
quit;

proc sql;
  create table temp2 as
  select *, count(*) as total, nz/count(*) as pz
  from temp1;
quit;

data &outdata;
    n=1;
    delete;
run;

%let i=&bcount;
  %do %until (&i>&ecount) ;
  proc sql noprint;
    create table count&i.a as
    select *, count(*) as ns&i
    from temp2
    group by z&i.st;
  quit;

  proc sql;
    create table count&i.b as
    select *, count(*) as ns&i.z
    from count&i.a
    group by z&i.st, z&i;
  quit;

  data count&i;
    set count&i.b;
    mmw=ns&i*pz/ns&i.z;
    if &z=&i;
  run;

  data &outdata;
    set &outdata count&i;
  run;

  proc means data=&outdata mean std n;
    class z&i.st;
    var mmw;
    where &z=&i;
  run;

  %let i=%eval(&i+1);
%end;

%mend MMW;

%MMW (indata=valid, outdata=MW.mmw);    /* specify input & output datasets */


*------------------------------------------------------------------------------;
* step 5 Check Balance;
*------------------------------------------------------------------------------;

%macro CHK (mydata, copre, bipre, capre);

proc contents noprint data= &mydata (keep=%treat)
  out=vars(keep= name);
run;

proc sql noprint;
  select name into :test separated by '='
  from vars;
quit;

/* check balance on logit score*/
%do i=&bcount %to &ecount;
    proc reg data=&mydata;
          model lgp_z&i = %treat/noint;
      weight mmw;
          test &test;
        run;
%end;

/* check balance on each pretreatment variable*/
data wpv (drop=n);
  n=1;
  delete;
run;

%let i=1;
  %do %until(not %length (%scan(&copre,&i)));
    proc reg data=&mydata;
      model %scan(&copre,&i)= %treat/noint;
          weight mmw;
          test &test;
        ods output testanova=cwpv&i;
        run;

    data wpv(drop= Model Test Source MS FValue Control);
      set wpv cwpv&i (where=(source='Numerator'));
      rename DF=%quote(%scan(&copre,&i));
    run;
  %let i=%eval(&i+1);
%end;

%let i=1;
  %do %until (not %length (%scan(&bipre,&i)));
    proc logistic data=&mydata ;
      model %scan(&bipre,&i)(EVENT='1')=%treat/noint;
          weight mmw;
          test &test;
        ods output teststmts=bwpv&i;
    run;

    data wpv (drop= Status Label WaldChiSq);
      set wpv bwpv&i(rename=(ProbChiSq=probF));
          rename DF=%quote(%scan(&bipre,&i));
        run;
  %let i=%eval(&i+1);
%end;

%let i=1;
  %do %until (not %length (%scan(&capre,&i)));
    proc freq data=&mydata;
      weight mmw;
      tables %scan(&capre,&i)*&z/chisq;
          output out=awpv&i pchi;
    run;

    data wpv (drop=_PCHI_);
      set wpv awpv&i(rename=(P_PCHI=probF));
          rename DF_PCHI=%quote(%scan(&capre,&i));
        run;
  %let i=%eval(&i+1);
%end;

proc transpose data=wpv out=eval1;
  copy probF;
run;

data eval (keep=probF _NAME_);
  set eval1;
  if probF>.05 then delete;
run;

proc print data=eval;
  var probF _NAME_;
run;

%mend CHK;

%macro treat;
%do n = &bcount %to &ecount;
  z&n
%end;
%mend treat;

%CHK (mydata=MW.mmw,
      copre=cc1mscale cwksesl,                     /* specify continuous pretreatment variables */
      bipre=dmmsc1msor dmmsses,                    /* specify binary pretreatment variables */
      capre=HMLANG race);                          /* specify categorical pretreatment variables */

*------------------------------------------------------------------------------;
* step 6 Estimate Treatment Effects;
*------------------------------------------------------------------------------;

/*naive analysis*/

proc glm data=MW.mmw;
  class ELLSERVC;
  model t2learn=ELLSERVC;
run;

/*weighted analysis*/

proc glm data=MW.mmw;
  class ELLSERVC;
  model t2learn=ELLSERVC;
  weight mmw;
run;

